@@ -1,5 +1,5 @@
 /*
- * this file is derived from "sixel" original version (2014-3-2)
+ * This file is derived from "sixel" original version (2014-3-2)
  * http://nanno.dip.jp/softlib/man/rlogin/sixel.tar.gz
  *
  * Initial developer of this file is kmiya@culti.
@@ -10,12 +10,12 @@
  *
  * He declares this is compatible with MIT/BSD/GPL.
  *
- * Hayaki Saito <user@zuse.jp> modified this and re-licensed
+ * Hayaki Saito <saitoha@me.com> modified this and re-licensed
  * it under the MIT license.
  *
  * The helper function hls2rgb is imported from Xterm pl#310.
  * This is originally written by Ross Combs.
- * Hayaki Saito <user@zuse.jp> slightly modified this.
+ * Hayaki Saito <saitoha@me.com> slightly modified this.
  *
  * -------------------------------------------------------------------------
  * Copyright 2013,2014 by Ross Combs
@@ -56,8 +56,8 @@
 # include <inttypes.h>
 #endif
 
+#include <sixel.h>
 #include "output.h"
-#include "sixel.h"
 
 #define RGB(r, g, b) (((r) << 16) + ((g) << 8) +  (b))
 
@@ -169,6 +169,7 @@ hls2rgb(int hue, int lum, int sat)
     return RGB(r * 255 / 100, g * 255 / 100, b * 255 / 100);
 }
 
+
 static unsigned char *
 sixel_getparams(unsigned char *p, int *param, int *len)
 {
@@ -206,49 +207,85 @@ sixel_getparams(unsigned char *p, int *param, int *len)
 
 /* convert sixel data into indexed pixel bytes and palette data */
 /* TODO: make "free" function as an argument */
-int
-sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
-             int                        /* in */  len,        /* size of sixel bytes */
-             unsigned char              /* out */ **pixels,   /* decoded pixels */
-             int                        /* out */ *pwidth,    /* image width */
-             int                        /* out */ *pheight,   /* image height */
-             unsigned char              /* out */ **palette,  /* ARGB palette */
-             int                        /* out */ *ncolors,   /* palette size (<= 256) */
-             sixel_allocator_function   /* out */ allocator)  /* malloc function */
+SIXELAPI SIXELSTATUS
+sixel_decode_raw(
+    unsigned char       /* in */  *p,         /* sixel bytes */
+    int                 /* in */  len,        /* size of sixel bytes */
+    unsigned char       /* out */ **pixels,   /* decoded pixels */
+    int                 /* out */ *pwidth,    /* image width */
+    int                 /* out */ *pheight,   /* image height */
+    unsigned char       /* out */ **palette,  /* ARGB palette */
+    int                 /* out */ *ncolors,   /* palette size (<= 256) */
+    sixel_allocator_t   /* in */  *allocator) /* allocator object */
 {
-    int n, i, r, g, b, sixel_vertical_mask, c;
-    int posision_x, posision_y;
-    int max_x, max_y;
-    int attributed_pan, attributed_pad;
-    int attributed_ph, attributed_pv;
-    int repeat_count, color_index;
+    SIXELSTATUS status = SIXEL_FALSE;
+    int n;
+    int i;
+    int r;
+    int g;
+    int b;
+    int sixel_vertical_mask;
+    int c;
+    int posision_x;
+    int posision_y;
+    int max_x;
+    int max_y;
+    int attributed_pan;
+    int attributed_pad;
+    int attributed_ph;
+    int attributed_pv;
+    int repeat_count;
+    int color_index;
     int max_color_index;
     int background_color_index;
     int param[10];
     int sixel_palet[SIXEL_PALETTE_MAX];
-    unsigned char *imbuf, *dmbuf;
-    int imsx, imsy;
-    int dmsx, dmsy;
+    unsigned char *imbuf;
+    unsigned char *dmbuf;
+    int imsx;
+    int imsy;
+    int dmsx;
+    int dmsy;
     int y;
 
     (void) len;
 
-    posision_x = posision_y = 0;
-    max_x = max_y = 0;
+    if (pixels == NULL) {
+        status = SIXEL_BAD_ARGUMENT;
+        goto end;
+    }
+
+    posision_x = 0;
+    posision_y = 0;
+    max_x = 0;
+    max_y = 0;
     attributed_pan = 2;
     attributed_pad = 1;
-    attributed_ph = attributed_pv = 0;
+    attributed_ph = 0;
+    attributed_pv = 0;
     repeat_count = 1;
     color_index = 15;
     max_color_index = 2;
     background_color_index = 0;
 
+    if (allocator == NULL) {
+        status = sixel_allocator_new(&allocator, NULL, NULL, NULL, NULL);
+        if (SIXEL_FAILED(status)) {
+            goto end;
+        }
+    } else {
+        sixel_allocator_ref(allocator);
+    }
+
+    *pixels = NULL;
     imsx = 2048;
     imsy = 2048;
-    imbuf = allocator(imsx * imsy);
-
+    imbuf = (unsigned char *)sixel_allocator_malloc(allocator, imsx * imsy);
     if (imbuf == NULL) {
-        return (-1);
+        sixel_helper_set_additional_message(
+            "sixel_deocde_raw: sixel_allocator_malloc() failed.");
+        status = SIXEL_BAD_ALLOCATION;
+        goto end;
     }
 
     for (n = 0; n < 16; n++) {
@@ -360,16 +397,19 @@ sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
             if (imsx < attributed_ph || imsy < attributed_pv) {
                 dmsx = imsx > attributed_ph ? imsx : attributed_ph;
                 dmsy = imsy > attributed_pv ? imsy : attributed_pv;
-                dmbuf = allocator(dmsx * dmsy);
+                dmbuf = (unsigned char *)sixel_allocator_malloc(allocator, dmsx * dmsy);
                 if (dmbuf == NULL) {
-                    free(imbuf);
-                    return (-1);
+                    sixel_allocator_free(allocator, imbuf);
+                    sixel_helper_set_additional_message(
+                        "sixel_deocde_raw: sixel_allocator_malloc() failed.");
+                    status = SIXEL_BAD_ALLOCATION;
+                    goto end;
                 }
                 memset(dmbuf, background_color_index, dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
                     memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
-                free(imbuf);
+                sixel_allocator_free(allocator, imbuf);
                 imsx = dmsx;
                 imsy = dmsy;
                 imbuf = dmbuf;
@@ -434,16 +474,16 @@ sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
 
                 dmsx = nx;
                 dmsy = ny;
-                dmbuf = allocator(dmsx * dmsy);
+                dmbuf = (unsigned char *)sixel_allocator_malloc(allocator, dmsx * dmsy);
                 if (dmbuf == NULL) {
-                    free(imbuf);
-                    return (-1);
+                    sixel_allocator_free(allocator, imbuf);
+                    goto end;
                 }
                 memset(dmbuf, background_color_index, dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
                     memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
-                free(imbuf);
+                sixel_allocator_free(allocator, imbuf);
                 imsx = dmsx;
                 imsy = dmsy;
                 imbuf = dmbuf;
@@ -517,9 +557,13 @@ sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
     if (imsx > max_x || imsy > max_y) {
         dmsx = max_x;
         dmsy = max_y;
-        if ((dmbuf = allocator(dmsx * dmsy)) == NULL) {
-            free(imbuf);
-            return (-1);
+        dmbuf = (unsigned char *)sixel_allocator_malloc(allocator, dmsx * dmsy);
+        if (dmbuf == NULL) {
+            sixel_allocator_free(allocator, imbuf);
+            sixel_helper_set_additional_message(
+                "sixel_deocde_raw: sixel_allocator_malloc() failed.");
+            status = SIXEL_BAD_ALLOCATION;
+            goto end;
         }
         for (y = 0; y < dmsy; ++y) {
             memcpy(dmbuf + dmsx * y, imbuf + imsx * y, dmsx);
@@ -530,18 +574,59 @@ sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
         imbuf = dmbuf;
     }
 
-    *pixels = imbuf;
+    *ncolors = max_color_index + 1;
+    *palette = (unsigned char *)sixel_allocator_malloc(allocator, *ncolors * 3);
+    if (palette == NULL) {
+        sixel_allocator_free(allocator, imbuf);
+        sixel_helper_set_additional_message(
+            "sixel_deocde_raw: sixel_allocator_malloc() failed.");
+        status = SIXEL_BAD_ALLOCATION;
+        goto end;
+    }
+    for (n = 0; n < *ncolors; ++n) {
+        (*palette)[n * 3 + 0] = sixel_palet[n] >> 16 & 0xff;
+        (*palette)[n * 3 + 1] = sixel_palet[n] >> 8 & 0xff;
+        (*palette)[n * 3 + 2] = sixel_palet[n] & 0xff;
+    }
+
     *pwidth = imsx;
     *pheight = imsy;
-    *ncolors = max_color_index + 1;
-    *palette = allocator(*ncolors * 4);
-    for (n = 0; n < *ncolors; ++n) {
-        (*palette)[n * 4 + 0] = sixel_palet[n] >> 16 & 0xff;
-        (*palette)[n * 4 + 1] = sixel_palet[n] >> 8 & 0xff;
-        (*palette)[n * 4 + 2] = sixel_palet[n] & 0xff;
-        (*palette)[n * 4 + 3] = 0xff;
+    *pixels = imbuf;
+
+    status = SIXEL_OK;
+
+end:
+    sixel_allocator_ref(allocator);
+    return status;
+}
+
+
+SIXELAPI SIXELSTATUS
+sixel_decode(unsigned char              /* in */  *p,         /* sixel bytes */
+             int                        /* in */  len,        /* size of sixel bytes */
+             unsigned char              /* out */ **pixels,   /* decoded pixels */
+             int                        /* out */ *pwidth,    /* image width */
+             int                        /* out */ *pheight,   /* image height */
+             unsigned char              /* out */ **palette,  /* ARGB palette */
+             int                        /* out */ *ncolors,   /* palette size (<= 256) */
+             sixel_allocator_function   /* in */  fn_malloc)  /* malloc function */
+{
+    SIXELSTATUS status = SIXEL_FALSE;
+    sixel_allocator_t *allocator = NULL;
+
+    status = sixel_allocator_new(&allocator, fn_malloc, NULL, NULL, NULL);
+    if (SIXEL_FAILED(status)) {
+        goto end;
     }
-    return 0;
+
+    status = sixel_decode_raw(p, len, pixels, pwidth, pheight, palette, ncolors, allocator);
+    if (SIXEL_FAILED(status)) {
+        goto end;
+    }
+
+end:
+    sixel_allocator_unref(allocator);
+    return status;
 }
 
 /* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
